////////////////////////////////////////////
//   Class ODES                           //
//                                        //
//   Original Class version: Nikos Kouvaris               // 
//   Creation Date 20/11/2005             //
//   GV version 1.0 , March 2012          //
////////////////////////////////////////////
//Revised .......................

#include "ODES.h"

//=========================== Private Methods ==================================================//
//==============================================================================================//
int ODES::bsstep(double y0[], double dydx[], double dt0, double yout[], double *dtnext){
	 int SMALLSTEP=0;
	static int nseq[IMAX+1]={0,2,4,6,8,12,16,24,32,48,64,96};
		
	double t=y0[0];
	for(int i=1;i<neq;i++) y[i]=y0[i];
	double h=dt0;
	double xsav=t;
	for (int i=1;i<neq;i++) {ysav[i]=y[i];dysav[i]=dydx[i];}
	int COMPLETE=0;
	do
	{
		for (int i=1;i<=IMAX;i++) 
		{
			mmid(ysav,dysav,xsav,h,nseq[i],yseq);
			double xest=h*h/(nseq[i]*nseq[i]);
			rzextr(i,xest,yseq,y,yerr);
			double errmax=0.0;
			for (int j=1;j<=ner;j++)
			{
				double tmp=fabs(yerr[j]);
				if (errmax<tmp) errmax=tmp;
			}
			//old if (errmax < BSAcc) : working failure for the  "rotation" project 
			if (errmax < BSAcc && errmax>=0)    //correction in 21/4/2008
			{
				t+= h;
				*dtnext = i==NUSE? h*SHRINK : i==NUSE-1? h*GROW : (h*nseq[NUSE-1])/nseq[i];
				COMPLETE=1; break; //return;
			}
		}
		if(COMPLETE) break;
		h*= 0.25;
		for (int i=1;i<=(IMAX-NUSE)/2;i++) h *= 0.5;
		SMALLSTEP=(h<BSMinStep);		
	}while(!SMALLSTEP);
	for(int i=1;i<neq;i++) yout[i]=y[i]; yout[0]=t;
	if(*dtnext>BSMaxStep) *dtnext=BSMaxStep;
	return (!SMALLSTEP);
}


void ODES::rzextr(int iest, double xest, double yest[], double yz[], double dy[]){
	double fx[NUSE+1];
	Bx[iest]=xest;
	if (iest == 1)	for (int j=1;j<neq;j++) {yz[j]=yest[j];	Bd[j][1]=yest[j];dy[j]=yest[j];}
	else 
	{	
		int m1=(iest < NUSE ? iest : NUSE);
		for (int k=1;k<m1;k++) fx[k+1]=Bx[iest-k]/xest;
		for (int j=1;j<neq;j++) 
		{
			double ddy;
			double yy=yest[j]; 
			double v=Bd[j][1];
			double c=yy;
			Bd[j][1]=yy;
			for (int k=2;k<=m1;k++) 
			{
				double b1=fx[k]*v;
				double b=b1-c;
				if(fabs(b)>BSAcc) { b=(c-v)/b;	ddy=c*b; c=b1*b;} 
				else ddy=v;
				if (k != m1) v=Bd[j][k];
				Bd[j][k]=ddy;
				yy += ddy;
			}
			dy[j]=ddy;
			yz[j]=yy;
		}
	}
}

void ODES::mmid(double y[],double dydt[],double t0, double dt0, int nstep, double yout[]){		
	
	double h=dt0/nstep;
	for (int i=1;i<neq;i++) 
	{
		ym[i]=y[i];
		ynn[i]=y[i]+h*dydt[i];
	}
	double t=t0+h;	derivs(t,ynn,yout);
	double h2=2.0*h;
	for (int n=2;n<=nstep;n++) 
	{
		for (int i=1;i<neq;i++) 
		{
			double swap=ym[i]+h2*yout[i];
			ym[i]=ynn[i]; ynn[i]=swap;
		}
		t += h;
		derivs(t,ynn,yout);
	}
	for (int i=1;i<neq;i++)	yout[i]=0.5*(ym[i]+ynn[i]+h*yout[i]);	
}




//============================ Public Methods ==================================================//
//==============================================================================================//



//------------------ constructor - destructor ---------------------------------------------------//
ODES::ODES(){    
    
}

//only for yse in BS
ODES::ODES(int numODES, void ODES_system(double , double y[], double dydt[]), double minstep, double maxstep, double acc, int n_error_check){         
    
	if(numODES<2) {BSINITIALIZED=0; return;} 
	derivs = ODES_system; if(derivs==NULL) {BSINITIALIZED=0; return;} 
	neq=numODES; 
	ner=n_error_check; if(ner<=0) ner=neq-1; //all vars
	acc=fabs(acc);
	if(acc>0.1||acc<1.0e-20) acc=0.001; BSAcc=acc;
	BSMinStep=minstep;	
	if(maxstep<minstep) maxstep=minstep*100; BSMaxStep=maxstep;
	
	if(BSINITIALIZED==1) return;
	
	Bx=new double[IMAX+1];
	int NN; if(neq>NUSE) NN=neq+1; else NN=NUSE+1;
	Bd=new double* [NN];
	for(int i=0;i<NN;i++) Bd[i]=new double[NN];
	
	y=new double [neq];
	ysav=new double [neq];
	dysav=new double [neq];
	yseq=new double [neq];
	yerr=new double [neq];
	ym=new double[neq];
	ynn=new double[neq];
	BSINITIALIZED=1;
}

ODES::~ODES(void){
	if(BSINITIALIZED==1){
	int NN; if(neq>NUSE) NN=neq+1; else NN=NUSE+1;
	delete [] Bx;
	for(int i=0;i<NN;i++) delete [] Bd[i];

	delete [] y;
	delete [] ysav;
	delete [] dysav;
	delete [] yseq;
	delete [] yerr;
	delete [] ym;
	delete [] ynn;
	}
	BSINITIALIZED=0;
}

void ODES::open(int numODES, void ODES_system(double , double y[], double dydt[]), double minstep, double maxstep, double acc, int n_error_check){         
    
	if(numODES<2) {BSINITIALIZED=0; return;} 
	derivs = ODES_system; if(derivs==NULL) {BSINITIALIZED=0; return;} 
	neq=numODES; 
	ner=n_error_check; if(ner<=0) ner=neq-1; //all vars
	acc=fabs(acc);
	if(acc>0.1||acc<1.0e-20) acc=0.001; BSAcc=acc;
	BSMinStep=minstep;	
	if(maxstep<minstep) maxstep=minstep*100; BSMaxStep=maxstep;
	
	if(BSINITIALIZED==1) return;
	
	Bx=new double[IMAX+1];
	int NN; if(neq>NUSE) NN=neq+1; else NN=NUSE+1;
	Bd=new double* [NN];
	for(int i=0;i<NN;i++) Bd[i]=new double[NN];
	
	y=new double [neq];
	ysav=new double [neq];
	dysav=new double [neq];
	yseq=new double [neq];
	yerr=new double [neq];
	ym=new double[neq];
	ynn=new double[neq];
	BSINITIALIZED=1;
}

int ODES::test(){ return 0; }


//------------------------  Bulirsch-Stoer integrator --------------------------------------//
int ODES::bsDTstep(double yin[], double DT, double *dt, double yout[]){	
    double *y=new double[neq];
	double *dydx=new double[neq]; 
	for(int i=0;i<neq;i++) y[i]=yin[i];
	double restT=DT,dT,dtn;
	if(DT<*dt) dT=DT; else dT=*dt;
	do
	{
		derivs(y[0],y,dydx);
		if(!bsstep(y,dydx,dT,yout,&dtn)) {delete [] y; 	delete [] dydx; return 0;}
		restT-=(yout[0]-y[0]);
		for(int i=0;i<neq;i++) y[i]=yout[i];
		if(restT>=dtn) {dT=dtn; *dt=dT;}
		else break; 
	}while(1);
	dT=restT;
	derivs(y[0],y,dydx);
	if(!bsstep(y,dydx,dT,yout,&dtn)) {delete [] y; 	delete [] dydx; return 0;}
	delete [] y; 	delete [] dydx;
	return 1;
}

