//DSYSTEM :  3-BODYPROBLEM PLANAR + DISSIPATION + VARMASS
//31/3/06  
//revised 20/2/07 - New FILENAMEs and delete unused calls  
//         Apr-2010 : version for VS2008
//         Mar-2012 : Regularization
//         Apr-2012 : Varmass Project : new version
//         May-2012 : choose varmass function

//Inertial Initial condition arrays
//  		BODY1	BODY2	BODY3 (used as the Star)
//X			1		2		from CM
//Y			3		4				
//VX		5		6				
//VY		7		8				



#include "dsys3BPvmdis.h"
//#include "iroe3bplane.h"


void dSystem3BPdsptv(double t, double y[],double dd[]);  //dissipative deqs

//========Global system variables ==================
double m1,m2,m3,m30,m20,m10; //masses
double Rm1,Rm2,Rm3; //masses for regularized setup
int VARMASS=0;  //flag for variable mass
int DISSIPATION=0;  //flag for Dissipation terms
double da=1;//coefficient1, SET ZERO for DISSIPATION=FALSE
double db=1.0e-5; //coefficient2
double vm1=0;  //use depends on the model, SET ZERO for VARMASS=FALSE
double vm2=10;
double massend;
double Ma1,Ma2; //anigmenes masses of system P1P3 and (P1P3)P2
double mu; //ratio m1/(m1+m3)
double Ener0,Lang0; //energy and angular momentum
//double zacc;



void InitializeSystem(double mass1, double mass2, double mass3, double dcoef1, double dcoef2, double vmcoef1, double vmcoef2)
{
	m1=mass1; m2=mass2; m3=mass3;
	vm1=vmcoef1;
	vm2=vmcoef2;
	db=dcoef1;
	da=dcoef2;
	mu=m1/(m1+m3);
	Ma1=m1*m3/(m1+m3);
	Ma2=(m1+m3)*m2/(m1+m2+m3);
	m10=m1; m20=m2; m30=m3;
	massend=m30*(1-vm1);
	if(vmcoef1==0||vmcoef2==0) VARMASS=0; else VARMASS=1;
	if(dcoef1==0) DISSIPATION=0; else DISSIPATION=1;
}



inline void varmassLin(double t) 
//n=0, linear decay
//vm1: decay factor
//vm2: ratio of mass loss
{
	m3=m30-vm2*t;
	//double massend=m30*(1-vm1);
	if(m3<massend) m3=massend;
	mu=m1/(m1+m3);
	Ma1=m1*m3/(m1+m3);
	Ma2=(m1+m3)*m2/(m1+m2+m3);	
}


inline void varmassSqrt(double t) 
//n=1/2, dot m =-a m^n
//vm1: decay factor
//vm2: ratio of mass loss
{
	m3=m30-vm2*sqrt(m30)*t+0.25*vm2*vm2*t*t;
	double massend=m30*(1-vm1);
	if(m3<massend) m3=massend;
	mu=m1/(m1+m3);
	Ma1=m1*m3/(m1+m3);
	Ma2=(m1+m3)*m2/(m1+m2+m3);	
}



inline void varmassExp(double t) 
//n=1,  Exp decay to zero
//vm1: decay factor
//vm2: ratio of mass loss
{
	m3=m30*exp(-vm2*t);
	double massend=m30*(1-vm1);
	if(m3<massend) m3=massend;
	mu=m1/(m1+m3);
	Ma1=m1*m3/(m1+m3);
	Ma2=(m1+m3)*m2/(m1+m2+m3);	
}


inline void varmassSqr(double t) 
//n=2, dot m =-a m^n
//vm1: decay factor
//vm2: ratio of mass loss
{
	m3=m30/(1+vm2*m30*t);
	double massend=m30*(1-vm1);
	if(m3<massend) m3=massend;
	mu=m1/(m1+m3);
	Ma1=m1*m3/(m1+m3);
	Ma2=(m1+m3)*m2/(m1+m2+m3);	
}


inline void varmassCub(double t) 
//n=3, dot m =-a m^n
//vm1: decay factor
//vm2: ratio of mass loss
{
	m3=m30/sqrt(1+2*vm2*m30*m30*t);
	double massend=m30*(1-vm1);
	if(m3<massend) m3=massend;
	mu=m1/(m1+m3);
	Ma1=m1*m3/(m1+m3);
	Ma2=(m1+m3)*m2/(m1+m2+m3);	
}




//=======================DERIV FUNCTION=============================================
//var y[i] : y[1]=x1,y[2]=x2,y[3]=y1, y[4]=y2,y[5]=vx1,....,y[8]=vy2
void dSystem3BPdsptv(double t, double y[],double dd[])
{
	if(VARMASS) varmassLin(t);
//3rd body
	double yx3=-(m1*y[1]+m2*y[2])/m3;
	double yy3=-(m1*y[3]+m2*y[4])/m3;
	//squares
	double r12=(y[1]-y[2])*(y[1]-y[2])+(y[3]-y[4])*(y[3]-y[4]);
	double r13=(y[1]- yx3)*(y[1]- yx3)+(y[3]- yy3)*(y[3]- yy3);
	double r23=(y[2]- yx3)*(y[2]- yx3)+(y[4]- yy3)*(y[4]- yy3);
	//squareroots
	double sr12=sqrt(r12);double sr13=sqrt(r13); double sr23=sqrt(r23); //squareroots
	//quartics
	double r123=r12*sr12; double r133=r13*sr13; double r233=r23*sr23;
	double r125=r123*r12; double r135=r133*r13; double r235=r233*r23;
	double sr133=sqrt(r133); double sr233=sqrt(r233);
	//Differential equations of 1st ORDER
	dd[1]=y[5];
	dd[2]=y[6];
	dd[3]=y[7];
	dd[4]=y[8];
	dd[5]=-m3*(y[1]-yx3)/r133-m2*(y[1]-y[2])/r123;
	dd[6]=-m3*(y[2]-yx3)/r233-m1*(y[2]-y[1])/r123;
	dd[7]=-m3*(y[3]-yy3)/r133-m2*(y[3]-y[4])/r123;
	dd[8]=-m3*(y[4]-yy3)/r233-m1*(y[4]-y[3])/r123;	
	if(DISSIPATION){
		dd[5]-=db*(y[5]+da*y[3]/sr133);
		dd[6]-=db*(y[6]+da*y[4]/sr233);
		dd[7]-=db*(y[7]-da*y[1]/sr133);
		dd[8]-=db*(y[8]-da*y[2]/sr233);
	}
}


//=======================DERIV FUNCTION=============================================
//var y[i] : y[1]=u1,y[2]=u2,y[3]=x3, y[4]=y3, y[5]=pu1, y[6]=pu2, y[7]=px3, y[8]=py3, y[9]=h, y[10]=t
void dSystem3BPreg(double t, double y[],double dd[])
{
	double irm12=1.0/(Rm1+Rm2);
	double xx=y[1]*y[1]-y[2]*y[2];
	double yy=2*y[1]*y[2];
	double x1=(Rm2*xx-Rm3*y[3])*irm12; double y1=(Rm2*yy-Rm3*y[4])*irm12;
	double x2=-(Rm1*xx+Rm3*y[3])*irm12; double y2=-(Rm1*yy+Rm3*y[4])*irm12;
	double x13=x1-y[3]; double x23=x2-y[3]; double y13=y1-y[4]; double y23=y2-y[4];
	double r=x13*x13+y13*y13; double ir133=1.0/(r*sqrt(r));
	       r=x23*x23+y23*y23; double ir233=1.0/(r*sqrt(r));
   double tD3x=x23*ir233-x13*ir133;
   double tD3y=y23*ir233-y13*ir133;
   double tE3x=Rm1*x13*ir133+Rm2*x23*ir233;
   double tE3y=Rm1*y13*ir133+Rm2*y23*ir233;
   r=y[1]*y[1]+y[2]*y[2];
   dd[1]=y[5];
   dd[2]=y[6];
   dd[3]=r*y[7];
   dd[4]=r*y[8];
   dd[5]=0.5*y[9]*y[1]+0.5*Rm3*r*(y[1]*tD3x+y[2]*tD3y);
   dd[6]=0.5*y[9]*y[2]+0.5*Rm3*r*(y[1]*tD3y-y[2]*tD3x);
   dd[7]=r*tE3x;
   dd[8]=r*tE3y;
   dd[9]=2*Rm3*(tD3x*(y[1]*y[5]-y[2]*y[6])+tD3y*(y[2]*y[5]+y[1]*y[6]));
   dd[10]=r;
}

	
void RegularizedToInertial(int collision, double yr[], double y[])
//collision mode : 1=P1+S, 2=P2+S, 3=P1+P2
//yr[1]=u1,yr[2]=u2,yr[3]=x3, yr[4]=y3, yr[5]=pu1, yr[6]=pu2, yr[7]=px3, yr[8]=py3, yr[9]=h, yr[10]=t
{
	double rm12=Rm1+Rm2;
	double rmc1=Rm1/rm12; double rmc2=Rm2/rm12; double rmc3=Rm3/rm12;
	double rx1,rx2,rx3,ry1,ry2,ry3,drx1,drx2,drx3,dry1,dry2,dry3;
	double  r=yr[1]*yr[1]+yr[2]*yr[2];
	double xx=yr[1]*yr[1]-yr[2]*yr[2]; 	double yy=2*yr[1]*yr[2];
	double dxx=2*(yr[1]*yr[5]-yr[2]*yr[6])/r; 	double dyy=2*(yr[2]*yr[5]+yr[1]*yr[6])/r;
	//body 3 (collisionless) 
	rx3=yr[3]; ry3=yr[4]; drx3=yr[7]; dry3=yr[8];
	//body 1
	rx1=-rmc3*rx3+rmc2*xx; ry1=-rmc3*ry3+rmc2*yy;
	drx1=-rmc3*drx3+rmc2*dxx; dry1=-rmc3*dry3+rmc2*dyy;
	//body 2
	rx2=-rmc3*rx3-rmc1*xx; ry2=-rmc3*ry3-rmc1*yy;
	drx2=-rmc3*drx3-rmc1*dxx; dry2=-rmc3*dry3-rmc1*dyy;
	//-----------------------------
	switch(collision){
		case 1: m1=Rm1; m2=Rm3; m3=Rm2;
				y[1]=rx1;  y[2]=rx3;  y[3]=ry1;  y[4]=ry3;
				y[5]=drx1; y[6]=drx3; y[7]=dry1; y[8]=dry3;
				break;
		case 2: m1=Rm3; m2=Rm2; m3=Rm1;
				y[1]=rx3;  y[2]=rx2;  y[3]=ry3;  y[4]=ry2;
				y[5]=drx3; y[6]=drx2; y[7]=dry3; y[8]=dry2;
				break;
		case 3: m1=Rm1; m2=Rm2; m3=Rm3;
				y[1]=rx1;  y[2]=rx2;  y[3]=ry1;  y[4]=ry2;
				y[5]=drx1; y[6]=drx2; y[7]=dry1; y[8]=dry2;
				break;
		default : return; 
	}
	y[0]=yr[10];
}

void InertialToRegularized(int collision, double y[], double yr[])
//collision mode : 1=P1+S, 2=P2+S, 3=P1+P2
{
	double rx1,rx2,rx3,ry1,ry2,ry3,drx1,drx2,drx3,dry1,dry2,dry3;
	double z[5];
	GetStarPos(y,z);
	switch(collision){
		case 1: Rm1=m1; rx1=y[1]; ry1=y[3]; drx1=y[5]; dry1=y[7];  
			    Rm2=m3; rx2=z[1]; ry2=z[2]; drx2=z[3]; dry2=z[4];
				Rm3=m2; rx3=y[2]; ry3=y[4]; drx3=y[6]; dry3=y[8]; 
				break;
		case 2: Rm1=m3; rx1=z[1]; ry1=z[2]; drx1=z[3]; dry1=z[4];  
			    Rm2=m2; rx2=y[2]; ry2=y[4]; drx2=y[6]; dry2=y[8];
				Rm3=m1; rx3=y[1]; ry3=y[3]; drx3=y[5]; dry3=y[7]; 
				break;
		case 3: Rm1=m1; rx1=y[1]; ry1=y[3]; drx1=y[5]; dry1=y[7];  
			    Rm2=m2; rx2=y[2]; ry2=y[4]; drx2=y[6]; dry2=y[8];
				Rm3=m3; rx3=z[1]; ry3=z[2]; drx3=z[3]; dry3=z[4]; 
				break;
		default : return; 
	}
	//initial conditions 
	//yr[1]=u1,yr[2]=u2,yr[3]=x3, yr[4]=y3, yr[5]=pu1, yr[6]=pu2, yr[7]=px3, yr[8]=py3, yr[9]=h, yr[10]=t
	double rxx=rx1-rx2; double ryy=ry1-ry2; double drxx=drx1-drx2; double dryy=dry1-dry2;
	double r=sqrt(rxx*rxx+ryy*ryy);
	yr[3]=rx3; yr[4]=ry3; yr[7]=drx3; yr[8]=dry3;
	yr[1]=ryy/sqrt(2*(r-rxx)); yr[2]=sqrt(0.5*(r-rxx));
	yr[5]=0.5*(yr[1]*drxx+yr[2]*dryy);
	yr[6]=0.5*(yr[1]*dryy-yr[2]*drxx);
	yr[9]=0.5*(drxx*drxx+dryy*dryy)-(Rm1+Rm2)/r;
	yr[10]=y[0];
	yr[0]=0;
}


//==========================================================================
//                      Integrals IN THE INERTIAL FRAME
//==========================================================================

double Energy(double y[])
{
	double z[5]; GetStarPos(y,z);
	double r12=(y[1]-y[2])*(y[1]-y[2])+(y[3]-y[4])*(y[3]-y[4]);
	double r13=(y[1]- z[1])*(y[1]- z[1])+(y[3]- z[2])*(y[3]- z[2]);
	double r23=(y[2]- z[1])*(y[2]- z[1])+(y[4]- z[2])*(y[4]- z[2]);
	r12=sqrt(r12); r13=sqrt(r13); r23=sqrt(r23); //squareroots
	
	double T=m1*(y[5]*y[5]+y[7]*y[7]);
	T+=m2*(y[6]*y[6]+y[8]*y[8]);
	T+=m3*(z[3]*z[3]+z[4]*z[4]);
	double V=-(m1*m2/r12+m1*m3/r13+m2*m3/r23);
	return 0.5*T+V;
}


double AngularMomentum(double y[])
{
	double L1=m1*(y[1]*y[7]-y[3]*y[5]);
	double L2=m2*(y[2]*y[8]-y[4]*y[6]);
	double z[5]; GetStarPos(y,z);
	double L3=m3*(z[1]*z[4]-z[2]*z[3]);
	double L=L1+L2+L3;
	return L;
}


void GetStarPos(double y[], double z[])
//y:systempos array , z[1..4]={x,y,xd,yd}
{
	z[0]=0.0;
	z[1]=-(m1*y[1]+m2*y[2])/m3;
	z[2]=-(m1*y[3]+m2*y[4])/m3;
	z[3]=-(m1*y[5]+m2*y[6])/m3;
	z[4]=-(m1*y[7]+m2*y[8])/m3;
}

void SetSystemPos(double X1[], double X2[], double Y[])
//get pos X1[4+1], X2[4+1] from bodies 1 and 2 and fill system pos data array Y[24+1]
{
	Y[0]=X1[0];
	Y[1]=X1[1]; Y[2]=X2[1]; //x 
	Y[3]=X1[2]; Y[4]=X2[2]; //y
	Y[5]=X1[3]; Y[6]=X2[3]; //xdot 
	Y[7]=X1[4]; Y[8]=X2[4]; //ydot
}


double GetPPdistance(double X[])
//X:system pos
{
	double r=(X[1]-X[2])*(X[1]-X[2])+(X[3]-X[4])*(X[3]-X[4]);
	return sqrt(r);
}

double GetSP1distance(double X[])
//X:system pos
{
	double Z[NEQ];
	GetStarPos(X, Z);
	double r=(X[1]-Z[1])*(X[1]-Z[1])+(X[3]-Z[2])*(X[3]-Z[2]);
	return sqrt(r);
}

double GetSP2distance(double X[])
//X:system pos
{
	double Z[NEQ];
	GetStarPos(X, Z);
	double r=(X[2]-Z[1])*(X[2]-Z[1])+(X[4]-Z[2])*(X[4]-Z[2]);
	return sqrt(r);
}


double GetPPdistance(double X1[], double X2[])
//X1,X2 planets pos
{
	double r=(X1[1]-X2[1])*(X1[1]-X2[1])+(X1[2]-X2[2])*(X1[2]-X2[2]);
	return sqrt(r);
}

//================tools=======================
void GetMasses(double *mass1, double *mass2, double *mass3)
{ 
	*mass1=m1; *mass2=m2; *mass3=m3;
}
void GetSysCoefs(double *d1,  double *d2, double *vmcf1,  double *vmcf2)
{
	*d1=db; *d2=da; *vmcf1=vm1; *vmcf2=vm2;
}
